/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.node.impl;

import javax.annotation.Nonnull;
import java.util.List;
import java.io.IOException;
import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.other.Type;
import it.tidalwave.semantic.event.RepositoryAdapter;
import it.tidalwave.semantic.event.RepositoryEvent;
import it.tidalwave.semantic.event.RepositoryListener;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class AllNameSpacesNode extends SemanticNode
  {
    @Nonnull
    private final Repository repository;

    @Nonnull
    private final NameSpaceChildFactory childFactory;

    /***************************************************************************
     *
     *
     **************************************************************************/
    private final RepositoryListener repositoryListener = new RepositoryAdapter()
      {
        @Override
        public void repositoryUpdated (@Nonnull final RepositoryEvent event)
          {
            childFactory.refresh();
          }
      };

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected static class NameSpaceChildFactory extends ChildFactory<String>
      {
        @Nonnull
        private final Repository repository;

        public NameSpaceChildFactory (@Nonnull final Repository repository)
          {
            this.repository = repository;
          }

        @Override
        @Nonnull
        protected Node createNodeForKey (@Nonnull final String namespace)
          {
            return new NamespaceNode(repository, namespace);
          }

        @Override
        protected boolean createKeys (@Nonnull final List<String> list)
          {
            final NameSpaceSegmenter nameSpaceSegmenter = new NameSpaceSegmenter();

            for (final Type type : repository.getAllTypes())
              {
                nameSpaceSegmenter.add(type.getId().stringValue());
              }

            list.addAll(nameSpaceSegmenter.getNameSpaces());

            return true;
          }

        public void refresh()
          {
            refresh(true);
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public AllNameSpacesNode (@Nonnull final Repository repository)
      {
        this(repository, new NameSpaceChildFactory(repository));
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private AllNameSpacesNode (@Nonnull final Repository repository,
                               @Nonnull final NameSpaceChildFactory childFactory)
      {
        super(Children.create(childFactory, true));
        this.repository = repository;
        this.childFactory = childFactory;
        repository.addListener(repositoryListener);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public void destroy()
      throws IOException
      {
        repository.removeListener(repositoryListener);
        super.destroy();
      }
  }
