/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.node.impl;

import javax.annotation.Nonnull;
import java.awt.Image;
import javax.swing.Action;
import org.openide.util.lookup.Lookups;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import it.tidalwave.semantic.node.SemanticNodeManager;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class SemanticNode extends AbstractNode
  {
    protected static final SemanticNodeManagerImpl semanticNodeManager = (SemanticNodeManagerImpl)SemanticNodeManager.Locator.findSemanticNodeManager();
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SemanticNode (@Nonnull final Children children,
                         @Nonnull final Object ... object)
      {
        super(children, Lookups.fixed(object));    
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SemanticNode (@Nonnull final Object ... object)
      {
        super(Children.LEAF, Lookups.fixed(object));    
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    @Nonnull 
    public final Image getIcon (final int type) 
      {
        return semanticNodeManager.findIcon(this);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    @Nonnull 
    public final Image getOpenedIcon (final int type) 
      {
        return semanticNodeManager.findIcon(this);
      }   

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    @Nonnull 
    public final Action[] getActions (final boolean context)
      {
        return semanticNodeManager.findNodeActions(getClass()).toArray(new Action[0]);
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public final Action getPreferredAction()
      {
        return semanticNodeManager.getPreferredNodeAction(getClass());
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean canCopy()
      {
        return true;
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean canCut()
      {
        return false;
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean canDestroy()
      {
        return false;
      }
    
    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean canRename()
      {
        return false;
      }
  }
