/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.node.impl;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import java.awt.Image;
import org.openide.nodes.Node;
import org.openide.util.Utilities;
import it.tidalwave.openide.IconResizer;
import it.tidalwave.openide.nodemanager.NodeManager;
import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Type;
import it.tidalwave.semantic.node.SemanticNodeManager;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class SemanticNodeManagerImpl extends NodeManager implements SemanticNodeManager
  {
    private final Image defaultTypeIcon;
    
    private final Image defaultEntityIcon;
    
    private final Map<String, Image> imageCacheByPath = new HashMap<String, Image>();
            
    private String localizedLanguage = "en"; // FIXME
        
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SemanticNodeManagerImpl()
      {
        defaultTypeIcon = findIcon("category.png");
        defaultEntityIcon = findIcon("tag.png");
        
//        configureActions(CategoryNode.class, "Catalog/Category");
//        configureActions(TagNode.class, "Catalog/Tag");
//
//        registerIcon(AllCategoriesNode.class, "category.png");
//        registerIcon(RootTagNode.class, "tag.png");
      }
     
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Node findRootNodes (@Nonnull final Repository repository)
      {
        return new AllNameSpacesNode(repository);
//        return new AllTypesNode(repository);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Node createEntityContainerNode (@Nonnull final Type type)
      {
        return new RootEntityNode(type);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public Image findIcon (@Nonnull final Node node)
      {
        final Type type = node.getLookup().lookup(Type.class);
        
        if (type != null)
          {
            return findIcon(type);
          }
        
        final Entity entity = node.getLookup().lookup(Entity.class);
        
        if (entity != null)
          {
            return findIcon(entity);
          }

        if ((node instanceof AllNameSpacesNode) || (node instanceof NamespaceNode) || (node instanceof AllTypesNode))
          {
            return defaultTypeIcon;
          }

        if (node instanceof RootEntityNode)
          {
            return defaultEntityIcon;
          }

        return super.findIcon(node);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull 
    public Image findIcon (@Nonnull final Type type)
      {
        Image icon = null;
        // FIXME findIcon(NbBundle.getMessage(CatalogNodeManagerImpl.class, "FN_CategoryIconPath", type.getQualifiedName()));

        if (icon == null)
          {
            icon = defaultTypeIcon;
          }

        return icon;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull 
    public Image findIcon (@Nonnull final Entity entity)
      {
        Image icon = null;
        // FIXME findIcon(NbBundle.getMessage(CatalogNodeManagerImpl.class, "FN_TagIconPath", entity.getCategory().getQualifiedName(), entity.getName()));

        if (icon == null)
          {
            icon = findIcon(entity.getType());
            
            if (icon == defaultTypeIcon)
              {
                icon = defaultEntityIcon;
              }
          }
        
        return icon;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public String findDisplayName (@Nonnull final Entity entity)
      {
        String displayName = entity.getDisplayName();
        String localizedName = null; // FIXME entity.findAttributeByName(localizedLanguage);
        
        if (localizedName != null)
          {
            displayName = displayName + " (" + localizedName + ")";    
          }
     
        return displayName;
      }
    
    /***************************************************************************
     *
     * This method is able to find icons in any module, so in order to provide
     * blueMarine with more icons it just needs to install new modules with icons
     * in the standard path "it/tidalwave/bluemarine/catalog".
     * 
     **************************************************************************/
    @Override
    @Nonnull
    protected Image findIcon (@Nonnull String path)
      {
        path = path.replace(' ', '_'); // Look at the old blueMarine code, there was a comment about a classloader bug
        Image image = imageCacheByPath.get(path);
        
        if (image == null)
          {
            image = IconResizer.scaledImage(Utilities.loadImage(path), ICON_SIZE);
            
            if (image == null)
              {
                image = super.findIcon(path);
              }
            
            imageCacheByPath.put(path, image);
          }
        
        return image;
      }
  }
